-- Local variables for runnning manually
DECLARE @RowLimit int
DECLARE @LastExportTime datetime
DECLARE @OnlyChanges bit
SET @OnlyChanges = 1
DECLARE @Version decimal
SET @Version = 2.0
DECLARE @TableName sysname
DECLARE @ServiceNowInstanceURL NVARCHAR(MAX)

--If we send NULL for LastExportTime from C#, which changes the datatype from datetime to nvarchar
--to correct that we are assigning the variable to internal variable with the datetime type
DECLARE @InternalLastExportTime datetime = @LastExportTime

-- Avoid locking by allowing result to contain uncommitted rows
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

-- Set the language to us_english so the datetime is consistent
SET LANGUAGE us_english

DECLARE @TenantUID varchar(40)

SET @TenantUID = (
	SELECT TOP 1 TenantUID
	FROM dbo.Tenant
	CROSS APPLY [dbo].[TableFilter_MT](TenantID)
	INNER JOIN dbo.TableIsMultiTenantEnabled() AS me
		ON me.MultiTenantEnabled = 1
)

IF @TableName IS NULL
	SET @TableName = 'dbo.ServiceNow_Applications' + ISNULL(N'_' + @TenantUID, '')

-- Get last ServiceNow export time - null if never run
IF @Version >= 2.0 AND @InternalLastExportTime IS NULL AND @OnlyChanges = 1
BEGIN
	SET @InternalLastExportTime = (
		SELECT TOP 1 bpe.StartTime
		FROM dbo.BatchProcessExecution AS bpe
		WHERE ((bpe.TenantUID IS NULL and @TenantUID IS NULL) OR bpe.TenantUID = @TenantUID)
		AND bpe.BatchProcessTypeID = 15 AND bpe.ReturnCode = 0
		AND EXISTS (	SELECT * 
						FROM dbo.BatchProcessExecutionData AS bped 
						WHERE bped.BatchProcessExecutionID = bpe.BatchProcessExecutionID
						AND bped.BatchProcessExecutionDataNameID = (SELECT BatchProcessExecutionDataNameID FROM dbo.BatchProcessExecutionDataName AS bpedn WHERE bpedn.Name = 'ServiceNowExportInstalledApplications')
						AND bped.DataValue = '1')
		AND EXISTS (	SELECT * 
						FROM dbo.BatchProcessExecutionData AS bped 
						WHERE bped.BatchProcessExecutionID = bpe.BatchProcessExecutionID
						AND bped.BatchProcessExecutionDataNameID = (SELECT BatchProcessExecutionDataNameID FROM dbo.BatchProcessExecutionDataName AS bpedn WHERE bpedn.Name = 'ServiceNowInstanceURL')
						AND (@ServiceNowInstanceURL IS NULL 
							OR LEFT(bped.DataValue, LEN(bped.DataValue) - PATINDEX('%[^/]%', REVERSE(bped.DataValue)) + 1) = @ServiceNowInstanceURL)
							)
		ORDER BY StartTime DESC
	)
END

-- Pre-calculate all application to computer links
IF OBJECT_ID('tempdb..#AppToComputer') IS NOT NULL DROP TABLE #AppToComputer
CREATE TABLE #AppToComputer (SoftwareTitleID int, ComplianceComputerID int, IsDeleted bit, PRIMARY KEY (SoftwareTitleID, ComplianceComputerID));
WITH links AS (
	SELECT	SoftwareTitleID,
			ComplianceComputerID,
			CASE WHEN hist.ComplianceHistoryTypeID = 5 THEN 1 ELSE 0 END AS IsDeleted
	FROM	(
				SELECT	SoftwareTitleID, ComplianceHistory.ComplianceComputerID, ComplianceHistoryTypeID, HistoryDate,
						ROW_NUMBER() OVER (PARTITION BY ComplianceHistory.ComplianceComputerID, SoftwareTitleID ORDER BY HistoryDate DESC) AS RowRank
				FROM	ComplianceHistory
				LEFT OUTER JOIN ComplianceComputer ON ComplianceComputer.ComplianceComputerID = ComplianceHistory.ComplianceComputerID
				LEFT OUTER JOIN ComplianceComputerTypeI18N ON ComplianceComputerTypeI18N.ComplianceComputerTypeID = ComplianceComputer.ComplianceComputerTypeID
				WHERE ComplianceHistory.ComplianceComputerID IS NOT NULL AND SoftwareTitleID IS NOT NULL AND ComplianceHistoryTypeID IN (4, 5) AND HistoryDate >= ISNULL(@InternalLastExportTime, HistoryDate) AND ( ComplianceComputerTypeI18N.ComplianceComputerTypeID IN (1,2,3,5) OR ComplianceHistoryTypeID = 5 ) 
			) AS hist
	WHERE	hist.RowRank = 1 AND @InternalLastExportTime IS NOT NULL AND @Version >= 2.0 -- If first time run then don't send diffs - version 1.0 does not support diffs
)
INSERT  #AppToComputer (SoftwareTitleID, ComplianceComputerID, IsDeleted)
SELECT  ISNULL(inst.SoftwareTitleID, links.SoftwareTitleID) AS SoftwareTitleID,
		ISNULL(inst.ComplianceComputerID, links.ComplianceComputerID) AS ComplianceComputerID,
		ISNULL(links.IsDeleted, 0) AS IsDeleted
FROM	(
			SELECT * FROM InstalledSoftware WHERE DiscoveryDate >= ISNULL(@InternalLastExportTime, DiscoveryDate)
		) AS inst
		FULL OUTER JOIN links ON links.SoftwareTitleID = inst.SoftwareTitleID AND links.ComplianceComputerID = inst.ComplianceComputerID

-- Limit rows returned if setting is present or provided row limit parameter not null
IF @RowLimit IS NULL OR @RowLimit = -1 BEGIN
	SET @RowLimit = 0x7FFFFFFF;
END

DECLARE @AdditionalServiceNowInstalledApplicationColumnSQL NVARCHAR(MAX) = N''
DECLARE @AdditionalServiceNowInstalledApplicationTableSQL NVARCHAR(MAX) = N''

SELECT @AdditionalServiceNowInstalledApplicationColumnSQL = SettingValue 
FROM dbo.ComplianceTenantSetting 
WHERE 
	SettingNameID = 170 -- AdditionalServiceNowInstalledApplicationColumnSQL
	
SELECT @AdditionalServiceNowInstalledApplicationTableSQL = SettingValue 
FROM dbo.ComplianceTenantSetting 
WHERE 
	SettingNameID = 171 -- AdditionalServiceNowInstalledApplicationTableSQL

-- Custom view results query
DECLARE @SQLString NVARCHAR(MAX)

SET @SQLString = N'
IF OBJECT_ID(''' + @TableName + N''') IS NOT NULL
	DROP TABLE ' + @TableName

EXECUTE sp_executesql @SQLString

/*
SET @SQLString = N'
SELECT  TOP(@RowLimit) results.*, ROW_NUMBER() OVER (ORDER BY (SELECT 1)) AS RowNumber INTO ' + @TableName + N'
FROM    (
*/
-- Custom view query for software titles
SELECT  ISNULL(SoftwareTitle.SoftwareTitleID, apps.SoftwareTitleID) AS [ApplicationID],
		SoftwareTitle.FullName AS [ApplicationName],
		SoftwareTitleVersion.VersionName AS [ApplicationVersion],
		SoftwareTitleClassificationI18N.DefaultValue AS [Classification],
		Recognition.FlexeraID AS [FlexeraID],
		ISNULL(NumberOfInstallationsCount.NumberOfInstallations, 0) AS [NumberOfInstallations],
		ISNULL(NumberOfLicensesCount.NumberOfLicenses, 0) AS [NumberOfLicenses],
		Product.ProductName AS [ProductName],
		SoftwareTitlePublisher.PublisherName AS [Publisher],
		[R2_bfd577cc81eb08f663b69a18109eada8_ApplicationToInstallation_InstalledSoftwareID],
		[R3_25e9fb8e5e8b8f49192383076433e948_InstallationToComputer_ComputerID],
		DiscoveredAt,
		LastScanned,
		LTRIM(RTRIM(ISNULL(SoftwareTitlePublisher.PublisherName, '''') + '' ''
			+ Product.ProductName + '' ''
			+ ISNULL(SoftwareTitleVersion.VersionName, ''''))) AS DisplayName,
		''FNMP'' AS DiscoveredBy,
		SerialNo--,
--		CAST(ISNULL(IsDeleted, 0) AS bit) AS IsDeleted
--		' + @AdditionalServiceNowInstalledApplicationColumnSQL + N'
FROM    (SELECT DISTINCT SoftwareTitleID FROM #AppToComputer) AS apps
		LEFT OUTER JOIN SoftwareTitle ON SoftwareTitle.SoftwareTitleID = apps.SoftwareTitleID
		-- Additional joins
		LEFT OUTER JOIN SoftwareTitleVersion ON SoftwareTitle.SoftwareTitleVersionID = SoftwareTitleVersion.SoftwareTitleVersionID
		LEFT OUTER JOIN SoftwareTitleClassificationI18N ON SoftwareTitleClassificationI18N.SoftwareTitleClassificationID = SoftwareTitle.SoftwareTitleClassificationID
		LEFT OUTER JOIN SoftwareRecognitionFlexeraID() AS Recognition ON SoftwareTitle.SoftwareTitleID = Recognition.SoftwareTitleID
		LEFT OUTER JOIN (SELECT COUNT(DISTINCT InstalledSoftware.InstalledSoftwareID) AS NumberOfInstallations, InstalledSoftware.SoftwareTitleID FROM InstalledSoftware GROUP BY InstalledSoftware.SoftwareTitleID) AS NumberOfInstallationsCount ON NumberOfInstallationsCount.SoftwareTitleID = SoftwareTitle.SoftwareTitleID
		LEFT OUTER JOIN (SELECT COUNT(DISTINCT SoftwareTitleLicense.SoftwareLicenseID) AS NumberOfLicenses, SoftwareTitleLicense.SoftwareTitleID FROM SoftwareTitleLicense GROUP BY SoftwareTitleLicense.SoftwareTitleID) AS NumberOfLicensesCount ON NumberOfLicensesCount.SoftwareTitleID = SoftwareTitle.SoftwareTitleID
		LEFT OUTER JOIN SoftwareTitleProduct AS Product ON SoftwareTitle.SoftwareTitleProductID = Product.SoftwareTitleProductID
		LEFT OUTER JOIN SoftwareTitleProduct ON SoftwareTitle.SoftwareTitleProductID = SoftwareTitleProduct.SoftwareTitleProductID LEFT OUTER JOIN SoftwareTitlePublisher ON SoftwareTitleProduct.SoftwareTitlePublisherID = SoftwareTitlePublisher.SoftwareTitlePublisherID
		-- Link from SoftwareTitle to Installations
		INNER JOIN (
			-- Custom view query for installed software
			SELECT  links.SoftwareTitleID,
					links.ComplianceComputerID,
					inst.InstalledSoftwareID AS [R2_bfd577cc81eb08f663b69a18109eada8_ApplicationToInstallation_InstalledSoftwareID],
					links.ComplianceComputerID AS R3_25e9fb8e5e8b8f49192383076433e948_InstallationToComputer_ComputerID,
					inst.DiscoveryDate AS DiscoveredAt,
					LastScanned,
					SerialNo,
					links.IsDeleted
			FROM    #AppToComputer AS links
			LEFT OUTER JOIN InstalledSoftware AS inst ON inst.SoftwareTitleID = links.SoftwareTitleID AND inst.ComplianceComputerID = links.ComplianceComputerID
			-- Additional joins
			-- Link from InstalledSoftware to Computer
			LEFT OUTER JOIN (
				-- Custom view query for computers
				SELECT  ComplianceComputerID,
						InventoryDate AS LastScanned,
						SerialNo
				FROM    ComplianceComputer
						-- Additional joins

			) AS comp ON comp.ComplianceComputerID = links.ComplianceComputerID
		) AS ApplicationToInstallation ON ApplicationToInstallation.SoftwareTitleID = apps.SoftwareTitleID
		' + @AdditionalServiceNowInstalledApplicationTableSQL + N'
/*
	) AS results
--'

EXECUTE sp_executesql @SQLString, N'@RowLimit INT', @RowLimit
*/