-- Local variables for runnning manually
DECLARE @RowLimit int
DECLARE @LastExportTime datetime
DECLARE @OnlyChanges bit
SET @OnlyChanges = 1
DECLARE @Version decimal
SET @Version = 2.0
DECLARE @TableName sysname
DECLARE @ServiceNowInstanceURL NVARCHAR(MAX)

--If we send NULL for LastExportTime from C#, which changes the datatype from datetime to nvarchar
--to correct that we are assigning the variable to internal variable with the datetime type
DECLARE @InternalLastExportTime datetime = @LastExportTime

-- Avoid locking by allowing result to contain uncommitted rows
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

-- Set the language to us_english so the datetime is consistent
SET LANGUAGE us_english

IF OBJECT_ID('tempdb..#ContractAllowed') IS NOT NULL
/*Then it exists*/
DROP TABLE #ContractAllowed

IF OBJECT_ID('tempdb..#AssetAllowed') IS NOT NULL
/*Then it exists*/
DROP TABLE #AssetAllowed

DECLARE @TenantUID varchar(40)
    
SET @TenantUID = (
	SELECT TOP 1 TenantUID
	FROM dbo.Tenant
	CROSS APPLY [dbo].[TableFilter_MT](TenantID)
	INNER JOIN dbo.TableIsMultiTenantEnabled() AS me
		ON me.MultiTenantEnabled = 1
)

IF @TableName IS NULL
	SET @TableName = 'dbo.ServiceNow_Contracts' + ISNULL(N'_' + @TenantUID, '')

-- Get last ServiceNow export time - null if never run
IF @InternalLastExportTime IS NULL AND @OnlyChanges = 1
BEGIN
	SET @InternalLastExportTime = (
		SELECT TOP 1 bpe.StartTime
		FROM dbo.BatchProcessExecution AS bpe
		WHERE ((bpe.TenantUID IS NULL and @TenantUID IS NULL) OR bpe.TenantUID = @TenantUID)
		AND bpe.BatchProcessTypeID = 15 AND bpe.ReturnCode = 0
		AND EXISTS (	SELECT * 
						FROM dbo.BatchProcessExecutionData AS bped 
						WHERE bped.BatchProcessExecutionID = bpe.BatchProcessExecutionID
						AND bped.BatchProcessExecutionDataNameID = (SELECT BatchProcessExecutionDataNameID FROM dbo.BatchProcessExecutionDataName AS bpedn WHERE bpedn.Name = 'ServiceNowExportContracts')
						AND bped.DataValue = '1')
		AND EXISTS (	SELECT * 
						FROM dbo.BatchProcessExecutionData AS bped 
						WHERE bped.BatchProcessExecutionID = bpe.BatchProcessExecutionID
						AND bped.BatchProcessExecutionDataNameID = (SELECT BatchProcessExecutionDataNameID FROM dbo.BatchProcessExecutionDataName AS bpedn WHERE bpedn.Name = 'ServiceNowInstanceURL')
						AND (@ServiceNowInstanceURL IS NULL 
							OR LEFT(bped.DataValue, LEN(bped.DataValue) - PATINDEX('%[^/]%', REVERSE(bped.DataValue)) + 1) = @ServiceNowInstanceURL)
							)
		ORDER BY StartTime DESC
	)
END

-- Pre-calculate contracts that user is allowed access - only contracts that have changed
CREATE TABLE #ContractAllowed (ContractID int PRIMARY KEY)
INSERT  #ContractAllowed SELECT allowed.ContractID FROM TableContractCurrentUser(NULL, NULL, NULL, NULL) AS allowed
LEFT OUTER JOIN (
	SELECT ContractID, HistoryDate, ROW_NUMBER() OVER (PARTITION BY ContractID ORDER BY HistoryDate DESC) AS Rank FROM ComplianceHistory
) AS hist ON hist.ContractID = allowed.ContractID AND hist.Rank = 1
WHERE @Version = 1.0 OR hist.HistoryDate IS NULL OR hist.HistoryDate >= ISNULL(@InternalLastExportTime, hist.HistoryDate)

-- Pre-calculate assets that user is allowed access
CREATE TABLE #AssetAllowed (AssetID int PRIMARY KEY)
INSERT  #AssetAllowed SELECT * FROM TableAssetCurrentUser(NULL, NULL, NULL, NULL)

-- Limit rows returned if setting is present or provided row limit parameter not null
IF @RowLimit IS NULL OR @RowLimit = -1 BEGIN
    SET @RowLimit = 0x7FFFFFFF;
END

DECLARE @AdditionalServiceNowContractColumnSQL NVARCHAR(MAX) = N''
DECLARE @AdditionalServiceNowContractTableSQL NVARCHAR(MAX) = N''

SELECT @AdditionalServiceNowContractColumnSQL = SettingValue 
FROM dbo.ComplianceTenantSetting 
WHERE 
	SettingNameID = 172 -- AdditionalServiceNowContractColumnSQL
	
SELECT @AdditionalServiceNowContractTableSQL = SettingValue 
FROM dbo.ComplianceTenantSetting 
WHERE 
	SettingNameID = 173 -- AdditionalServiceNowContractTableSQL

-- Custom view results query
DECLARE @SQLString NVARCHAR(MAX)

SET @SQLString = N'
IF OBJECT_ID(''' + @TableName + N''') IS NOT NULL
	DROP TABLE ' + @TableName

EXECUTE sp_executesql @SQLString

/*
SET @SQLString = N'
SELECT  TOP(@RowLimit) results.*, ROW_NUMBER() OVER (ORDER BY (SELECT 1)) AS RowNumber INTO ' + @TableName + N'
FROM    (
-- Custom view query for contracts
*/
SELECT  Contract.ContractID AS [ContractID],
		Contract.ContractName AS [ContractName],
		Contract.ContractNo AS [ContractNumber],
		ContractStatusI18N.DefaultValue AS [ContractStatus],
		ContractTypeI18N.ContractTypeDefaultValue AS [ContractType],
		Contract.EndDate AS [EndDate],
		Contract.StartDate AS [StartDate],
		v.VendorName AS [Vendor],
		Contract.MasterContractID AS [MasterContractID],
		[R2_6735c975da30de77bb701f0f3f3ccfa6_ContractToAsset_AssetID],
		[R2_6735c975da30de77bb701f0f3f3ccfa6_ContractToAsset_AssetType],
		CAST(0 AS bit) AS IsDeleted -- Placeholder for now
--		' + @AdditionalServiceNowContractColumnSQL + N'
FROM    Contract
		-- Filter
		INNER JOIN #ContractAllowed AS filter ON filter.ContractID = Contract.ContractID
		-- Additional joins
		LEFT OUTER JOIN dbo.ContractStatusI18N ON ContractStatusI18N.ContractStatusID = Contract.ContractStatusID
		LEFT OUTER JOIN dbo.ContractTypeI18N ON ContractTypeI18N.ContractTypeID = Contract.ContractTypeID
		LEFT OUTER JOIN dbo.Vendor AS v ON v.VendorID = Contract.VendorID
		-- Link from Contract to Assets via AssetContract table
		LEFT OUTER JOIN (
			SELECT  AssetContract.ContractID,
					Asset.*
			FROM    AssetContract
			INNER JOIN (
				-- Custom view query for assets
				SELECT  Asset.AssetID AS [R2_6735c975da30de77bb701f0f3f3ccfa6_ContractToAsset_AssetID],
						AssetTypeI18N.AssetTypeName AS [R2_6735c975da30de77bb701f0f3f3ccfa6_ContractToAsset_AssetType]
				FROM    Asset
				-- Filter
				INNER JOIN #AssetAllowed AS filter ON filter.AssetID = Asset.AssetID
				-- Additional joins
				LEFT OUTER JOIN AssetTypeI18N ON AssetTypeI18N.AssetTypeID = Asset.AssetTypeID
			) AS Asset ON Asset.R2_6735c975da30de77bb701f0f3f3ccfa6_ContractToAsset_AssetID = AssetContract.AssetID
		) AS ContractToAsset ON ContractToAsset.ContractID = Contract.ContractID
--		' + @AdditionalServiceNowContractTableSQL + N'
/*
	) AS results
'

EXECUTE sp_executesql @SQLString, N'@RowLimit INT', @RowLimit
*/
