-- Local variables for runnning manually
DECLARE @RowLimit int
DECLARE @LastExportTime datetime
DECLARE @OnlyChanges bit
SET @OnlyChanges = 1
DECLARE @Version decimal
SET @Version = 2.0
DECLARE @TableName sysname
DECLARE @ServiceNowInstanceURL NVARCHAR(MAX)

--If we send NULL for LastExportTime from C#, which changes the datatype from datetime to nvarchar
--to correct that we are assigning the variable to internal variable with the datetime type
DECLARE @InternalLastExportTime datetime = @LastExportTime

-- Avoid locking by allowing result to contain uncommitted rows
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

-- Set the language to us_english so the datetime is consistent
SET LANGUAGE us_english

IF OBJECT_ID('tempdb..#ComplianceComputerAllowed') IS NOT NULL
/*Then it exists*/
DROP TABLE #ComplianceComputerAllowed

IF OBJECT_ID('tempdb..#InstalledSoftwareAllowedComputer') IS NOT NULL
/*Then it exists*/
DROP TABLE #InstalledSoftwareAllowedComputer

IF OBJECT_ID('tempdb..#InstalledSoftwareAllowed') IS NOT NULL
/*Then it exists*/
DROP TABLE #InstalledSoftwareAllowed

IF OBJECT_ID('tempdb..#SoftwareLicenseAllocationAllowed') IS NOT NULL
/*Then it exists*/
DROP TABLE #SoftwareLicenseAllocationAllowed

IF OBJECT_ID('tempdb..#VirtualMachineAllowed') IS NOT NULL
/*Then it exists*/
DROP TABLE #VirtualMachineAllowed

IF OBJECT_ID('tempdb..#AssetAllowed') IS NOT NULL
/*Then it exists*/
DROP TABLE #AssetAllowed

DECLARE @TenantUID varchar(40)

SET @TenantUID = (
	SELECT TOP 1 TenantUID
	FROM dbo.Tenant
	CROSS APPLY [dbo].[TableFilter_MT](TenantID)
	INNER JOIN dbo.TableIsMultiTenantEnabled() AS me
		ON me.MultiTenantEnabled = 1
)

IF @TableName IS NULL
	SET @TableName = 'dbo.ServiceNow_Inventory' + ISNULL(N'_' + @TenantUID, '')

-- Get last ServiceNow export time - null if never run
IF @InternalLastExportTime IS NULL AND @OnlyChanges = 1
BEGIN
	SET @InternalLastExportTime = (
		SELECT TOP 1 bpe.StartTime
		FROM dbo.BatchProcessExecution AS bpe
		WHERE ((bpe.TenantUID IS NULL and @TenantUID IS NULL) OR bpe.TenantUID = @TenantUID)
		AND bpe.BatchProcessTypeID = 15 AND bpe.ReturnCode = 0
		AND EXISTS (	SELECT * 
						FROM dbo.BatchProcessExecutionData AS bped 
						WHERE bped.BatchProcessExecutionID = bpe.BatchProcessExecutionID
						AND bped.BatchProcessExecutionDataNameID = (SELECT BatchProcessExecutionDataNameID FROM dbo.BatchProcessExecutionDataName AS bpedn WHERE bpedn.Name = 'ServiceNowExportHardwareInventory')
						AND bped.DataValue = '1')
		AND EXISTS (	SELECT * 
						FROM dbo.BatchProcessExecutionData AS bped 
						WHERE bped.BatchProcessExecutionID = bpe.BatchProcessExecutionID
						AND bped.BatchProcessExecutionDataNameID = (SELECT BatchProcessExecutionDataNameID FROM dbo.BatchProcessExecutionDataName AS bpedn WHERE bpedn.Name = 'ServiceNowInstanceURL')
						AND (@ServiceNowInstanceURL IS NULL 
							OR LEFT(bped.DataValue, LEN(bped.DataValue) - PATINDEX('%[^/]%', REVERSE(bped.DataValue)) + 1) = @ServiceNowInstanceURL)
							)
		ORDER BY StartTime DESC
	)
END

-- Pre-calculate computers that user is allowed access - only computers that have changed
CREATE TABLE #ComplianceComputerAllowed (ComplianceComputerID int PRIMARY KEY, serialNo nvarchar(max) , IsDeleted bit);

--add the deleted links
WITH links AS (
	SELECT	
			ComplianceComputerID,
			OldValue,
			CASE WHEN hist.ComplianceHistoryTypeID = 22 THEN 1 ELSE 0 END AS IsDeleted
	FROM	(
				SELECT	 ComplianceComputerID, OldValue, ComplianceHistoryTypeID, HistoryDate,
						ROW_NUMBER() OVER (PARTITION BY ComplianceComputerID ORDER BY HistoryDate DESC) AS RowRank
				FROM	ComplianceHistory
				WHERE	ComplianceComputerID IS NOT NULL AND FieldName  = 'SerialNumber' AND OldValue IS NOT NULL AND ComplianceHistoryTypeID IN (22)  AND HistoryDate >= ISNULL(@InternalLastExportTime, HistoryDate)
			) AS hist
	WHERE	hist.RowRank = 1 AND @InternalLastExportTime IS NOT NULL AND @Version >= 2.0 -- If first time run then don't send diffs - version 1.0 does not support diffs
)
INSERT INTO #ComplianceComputerAllowed
SELECT ComplianceComputerID,OldValue, IsDeleted  FROM links

INSERT  #ComplianceComputerAllowed 
SELECT allowed.ComplianceComputerID, NULL, 0 
FROM TableComplianceComputerCurrentUser(NULL, NULL, NULL, NULL) AS allowed
JOIN ComplianceComputerActive ccna 
ON ccna.ComplianceComputerID = allowed.ComplianceComputerID  
LEFT OUTER JOIN (
	SELECT ComplianceComputerID, HistoryDate, ROW_NUMBER() OVER (PARTITION BY ComplianceComputerID ORDER BY HistoryDate DESC) AS Rank FROM ComplianceHistory
) AS hist ON hist.ComplianceComputerID = allowed.ComplianceComputerID AND hist.Rank = 1
WHERE @Version = 1.0 OR hist.HistoryDate IS NULL OR hist.HistoryDate >= ISNULL(@InternalLastExportTime, hist.HistoryDate)

-- Pre-calculate installed software that user is allowed access
CREATE TABLE #InstalledSoftwareAllowedComputer (ComplianceComputerID int PRIMARY KEY)

CREATE TABLE #InstalledSoftwareAllowed (InstalledSoftwareID int PRIMARY KEY)

IF (SELECT HasAccess FROM dbo.RightsHasAccess('CMApplications', 'read')) = 1
BEGIN
	INSERT	#InstalledSoftwareAllowedComputer
	SELECT	ComplianceComputerID
	FROM	TableComplianceComputerCurrentUser(NULL, NULL, NULL, NULL)

	INSERT  #InstalledSoftwareAllowed
	SELECT  ins.InstalledSoftwareID FROM InstalledSoftware AS ins
	JOIN    #InstalledSoftwareAllowedComputer AS cc ON cc.ComplianceComputerID = ins.ComplianceComputerID
END

-- Pre-calculate software license allocations that user is allowed access
CREATE TABLE #SoftwareLicenseAllocationAllowed (SoftwareLicenseAllocationID int PRIMARY KEY)
INSERT  #SoftwareLicenseAllocationAllowed
-- User allocations for users visible to the current operator
--(
	SELECT	sla.SoftwareLicenseAllocationID FROM SoftwareLicenseAllocation AS sla
	JOIN	SoftwareLicenseCurrentUser AS sl ON sl.SoftwareLicenseID = sla.SoftwareLicenseID
	JOIN	dbo.ComplianceUserCurrentUserScoped AS cucus ON (cucus.ComplianceUserID = sla.ComplianceUserID)
--)
UNION ALL
(
	-- Computer allocations for computers visible to the current operator
	SELECT	sla.SoftwareLicenseAllocationID FROM SoftwareLicenseAllocationNotArchived AS sla
	JOIN	SoftwareLicenseCurrentUser AS sl ON sl.SoftwareLicenseID = sla.SoftwareLicenseID
	JOIN	dbo.ComplianceComputerCurrentUserScoped AS cccus ON (cccus.ComplianceComputerID = sla.ComplianceComputerID)
)

-- Pre-calculate virtual machines that user is allowed access
CREATE TABLE #VirtualMachineAllowed (ComplianceComputerID int PRIMARY KEY, VMHostID int, VMType nvarchar(max))
INSERT  #VirtualMachineAllowed
SELECT  vm.ComplianceComputerID, vm.HostComplianceComputerID, VMTypeI18N.DefaultValue FROM VirtualMachineInfo AS vm
JOIN    TableComplianceComputerCurrentUser(NULL, NULL, NULL, NULL) AS cc ON cc.ComplianceComputerID = vm.ComplianceComputerID
LEFT OUTER JOIN VMTypeI18N as vmTypeI18N ON vm.VMTypeID = vmTypeI18N.VMTypeID


-- Pre-calculate assets that user is allowed access
CREATE TABLE #AssetAllowed (AssetID int PRIMARY KEY)
INSERT  #AssetAllowed SELECT * FROM TableAssetCurrentUser(NULL, NULL, NULL, NULL)

-- Limit rows returned if setting is present or provided row limit parameter not null
IF @RowLimit IS NULL OR @RowLimit = -1 BEGIN
    SET @RowLimit = 0x7FFFFFFF;
END

-- Custom view results query
DECLARE @AdditionalServiceNowHardwareInventoryColumnSQL NVARCHAR(MAX) = N''
DECLARE @AdditionalServiceNowHardwareInventoryTableSQL NVARCHAR(MAX) = N''

SELECT @AdditionalServiceNowHardwareInventoryColumnSQL = SettingValue
FROM dbo.ComplianceTenantSetting 
WHERE 
	SettingNameID = 168 -- AdditionalServiceNowHardwareInventoryColumnSQL
	
SELECT @AdditionalServiceNowHardwareInventoryTableSQL = SettingValue
FROM dbo.ComplianceTenantSetting 
WHERE 
	SettingNameID = 169 -- AdditionalServiceNowHardwareInventoryTableSQL

-- Custom view results query
DECLARE @SQLString NVARCHAR(MAX)

SET @SQLString = N'
IF OBJECT_ID(''' + @TableName + N''') IS NOT NULL
	DROP TABLE ' + @TableName

EXECUTE sp_executesql @SQLString

/*
SET @SQLString = N'
SELECT  TOP(@RowLimit) results.*, ROW_NUMBER() OVER (ORDER BY (SELECT 1)) AS RowNumber INTO ' + @TableName + N'
FROM    (
    -- Custom view query for computers

*/
    SELECT  ISNULL(ComplianceComputer.ComplianceComputerID, filter.ComplianceComputerID) AS [ComputerID],
            ComplianceComputer.ComputerName AS [ComputerName],
            AssetComplianceStatusI18N.StatusDefaultValue AS [AssetComplianceStatus],
            ComputerChassisTypeI18N.DefaultValue AS [ChassisType],
            ComplianceComputerStatusI18N.DefaultValue AS [ComputerStatus],
            ComplianceComputerTypeI18N.DefaultValue AS [ComputerType],
            cd.QualifiedName AS [Domain],
            flatd.FlatName AS [FlatDomain],
            ComplianceComputer.InventoryDate AS [InventoryDate],
            ComplianceComputerInventorySourceTypeI18N.DefaultValue AS [InventorySourceType],
            ComplianceComputer.IPAddress AS [IPAddress],
            ComplianceComputer.MACAddress AS [MACAddress],
            ComplianceComputer.Manufacturer AS [Manufacturer],
            ComplianceComputer.MaxClockSpeed AS [MaxClockSpeed],
            ComplianceComputer.ModelNo AS [ModelNo],
            ComplianceComputer.NumberOfCores AS [NumberOfCores],
            ComplianceComputer.NumberOfDisplayAdapters AS [NumberOfDisplayAdapters],
            ComplianceComputer.NumberOfHardDrives AS [NumberOfHardDrives],
            ISNULL(NumberOfInstallationsCount.NumberOfInstallations, 0) AS [NumberOfInstallations],
            ComplianceComputer.NumberOfNetworkCards AS [NumberOfNetworkCards],
            ComplianceComputer.NumberOfProcessors AS [NumberOfProcessors],
            ComplianceComputer.NumberOfSockets AS [NumberOfSockets],
            ISNULL(NumberOfSoftwareLicenseAllocationsCount.NumberOfSoftwareLicenseAllocations, 0) AS [NumberOfSoftwareLicenseAllocations],
            ComplianceComputer.NumberOfLogicalProcessors AS [NumberOfThreads],
            ISNULL(NumberOfVirtualMachinesCount.NumberOfVirtualMachines, 0) AS [NumberOfVirtualMachines],
            ComplianceComputer.OperatingSystem AS [OperatingSystem],
            ComplianceComputer.PartialNumberOfProcessors AS [PartialNumberOfProcessors],
            ComplianceComputer.ProcessorType AS [ProcessorType],
            ISNULL(ComplianceComputer.SerialNo, filter.SerialNo ) AS [SerialNo],
            ComplianceComputer.TotalDiskSpace  AS [TotalDiskSpace],
			ComplianceComputer.TotalMemory  AS [TotalMemory],
            [R2_6b0a0fa79a218b583be15d4d7788286f_ComputerToAsset_AssetID],
            [R2_6b0a0fa79a218b583be15d4d7788286f_ComputerToAsset_AssetType],
			last_user.UserName AS LastLoggedInUser,
			calc_user.UserName AS CalculatedUser,
			ComplianceComputer.CreationDate AS DiscoveredDate,
			last_con.SourceType AS InventorySource,
			last_con.ConnectionName AS InventoryConnectionName,
			CAST(filter.IsDeleted AS bit) AS IsDeleted,
			VMAllowed.VMType,
			VMAllowed.VMHostID
			' + @AdditionalServiceNowHardwareInventoryColumnSQL + N'
    FROM     #ComplianceComputerAllowed AS filter 
            LEFT OUTER JOIN ComplianceComputer ON filter.ComplianceComputerID = ComplianceComputer.ComplianceComputerID
            -- Additional joins
            LEFT OUTER JOIN dbo.AssetComplianceStatusI18N ON AssetComplianceStatusI18N.AssetComplianceStatusID = ComplianceComputer.AssetComplianceStatusID
            LEFT OUTER JOIN ComputerChassisTypeI18n ON ComputerChassisTypeI18n.ChassisTypeID = ComplianceComputer.ChassisTypeID
            LEFT OUTER JOIN dbo.ComplianceComputerStatusI18N ON ComplianceComputerStatusI18N.ComplianceComputerStatusID = ComplianceComputer.ComplianceComputerStatusID
            LEFT OUTER JOIN dbo.ComplianceComputerTypeI18N ON ComplianceComputerTypeI18N.ComplianceComputerTypeID = ComplianceComputer.ComplianceComputerTypeID
            LEFT OUTER JOIN ComplianceDomain AS cd ON cd.ComplianceDomainID = ComplianceComputer.ComplianceDomainID
            LEFT OUTER JOIN dbo.ComplianceDomain AS flatd ON flatd.ComplianceDomainID = [ComplianceComputer].ComplianceDomainID
            LEFT OUTER JOIN dbo.ComplianceComputerInventorySourceTypeI18N ON ComplianceComputerInventorySourceTypeI18N.ComplianceComputerInventorySourceTypeID = ComplianceComputer.ComplianceComputerInventorySourceTypeID
            LEFT OUTER JOIN (SELECT COUNT(DISTINCT InstalledSoftware.InstalledSoftwareID) AS NumberOfInstallations, InstalledSoftware.ComplianceComputerID FROM InstalledSoftware INNER JOIN #InstalledSoftwareAllowed AS InstalledSoftware_filter ON InstalledSoftware_filter.InstalledSoftwareID = InstalledSoftware.InstalledSoftwareID GROUP BY InstalledSoftware.ComplianceComputerID) AS NumberOfInstallationsCount ON NumberOfInstallationsCount.ComplianceComputerID = ComplianceComputer.ComplianceComputerID
            LEFT OUTER JOIN (SELECT COUNT(DISTINCT SoftwareLicenseAllocation.SoftwareLicenseAllocationID) AS NumberOfSoftwareLicenseAllocations, SoftwareLicenseAllocation.ComplianceComputerID FROM SoftwareLicenseAllocation INNER JOIN #SoftwareLicenseAllocationAllowed AS SoftwareLicenseAllocation_filter ON SoftwareLicenseAllocation_filter.SoftwareLicenseAllocationID = SoftwareLicenseAllocation.SoftwareLicenseAllocationID GROUP BY SoftwareLicenseAllocation.ComplianceComputerID) AS NumberOfSoftwareLicenseAllocationsCount ON NumberOfSoftwareLicenseAllocationsCount.ComplianceComputerID = ComplianceComputer.ComplianceComputerID
            LEFT OUTER JOIN (SELECT COUNT(DISTINCT VirtualMachine.ComplianceComputerID) AS NumberOfVirtualMachines, VirtualMachine.HostComplianceComputerID FROM VirtualMachine INNER JOIN #VirtualMachineAllowed AS VirtualMachine_filter ON VirtualMachine_filter.ComplianceComputerID = VirtualMachine.ComplianceComputerID GROUP BY VirtualMachine.HostComplianceComputerID) AS NumberOfVirtualMachinesCount ON NumberOfVirtualMachinesCount.HostComplianceComputerID = ComplianceComputer.ComplianceComputerID
			LEFT OUTER JOIN  #VirtualMachineAllowed as VMAllowed ON VMAllowed.ComplianceComputerID = ComplianceComputer.ComplianceComputerID
            -- Link from Computer to Asset
            LEFT OUTER JOIN (
				-- Custom view query for assets
				SELECT  Asset.AssetID AS [R2_6b0a0fa79a218b583be15d4d7788286f_ComputerToAsset_AssetID],
						AssetTypeI18N.AssetTypeName AS [R2_6b0a0fa79a218b583be15d4d7788286f_ComputerToAsset_AssetType]
				FROM    Asset
				-- Filter
				INNER JOIN #AssetAllowed AS filter ON filter.AssetID = Asset.AssetID
				-- Additional joins
				LEFT OUTER JOIN AssetTypeI18N ON AssetTypeI18N.AssetTypeID = Asset.AssetTypeID
            ) AS ComputerToAsset ON ComputerToAsset.R2_6b0a0fa79a218b583be15d4d7788286f_ComputerToAsset_AssetID = ComplianceComputer.AssetID
			-- Get logged on and calculated users
			LEFT OUTER JOIN ComplianceUser AS last_user ON last_user.ComplianceUserID = ComplianceComputer.ComplianceUserID
			LEFT OUTER JOIN ComplianceUser AS calc_user ON calc_user.ComplianceUserID = ISNULL(ComplianceComputer.AssignedUserID, ComplianceComputer.CalculatedUserID)
			-- Get latest connection that inventoried computer
			LEFT OUTER JOIN (
				SELECT	ic.ComplianceComputerID,
						con.SourceType,
						con.ConnectionName,
						ROW_NUMBER() OVER (PARTITION BY ic.ComplianceComputerID ORDER BY ic.LastSuccessfulInventoryDate DESC, ic.ComplianceConnectionID) AS RowRank
				FROM	ImportedComputer AS ic
						INNER JOIN ComplianceConnection AS con ON con.ComplianceConnectionID = ic.ComplianceConnectionID
			) AS last_con ON last_con.ComplianceComputerID = ComplianceComputer.ComplianceComputerID AND RowRank = 1
--			' + @AdditionalServiceNowHardwareInventoryTableSQL + N'
			--we want all the records with any computer type to be sent to ServiceNow. For deleted record type is set to NULL hence the OR condition
			WHERE ComplianceComputerTypeI18N.ComplianceComputerTypeID IN (1,2,3,5) OR filter.IsDeleted = 1
/*
        ) AS results order by results.IsDeleted desc
'

EXECUTE sp_executesql @SQLString, N'@RowLimit INT', @RowLimit
*/